"""Define the main application."""
import argparse
import json
import logging

from aiohttp import web
from asyncio_mqtt import Client, MqttError

_LOGGER = logging.getLogger("ecowitt2mqtt")

DEFAULT_AIOHTTP_ENDPOINT = "/data/report"
DEFAULT_AIOHTTP_PORT = 8080
DEFAULT_LOG_LEVEL_STRING = "INFO"
DEFAULT_MQTT_PORT = 1883


def get_arguments() -> argparse.Namespace:
    """Get passed-in arguments."""
    parser = argparse.ArgumentParser(
        description="Send data from Ecowitt devices to an MQTT broker"
    )
    parser.add_argument(
        "--mqtt-broker",
        action="store",
        required=True,
        type=str,
        help="The hostname or IP address of the MQTT broker",
    )
    parser.add_argument(
        "--mqtt-topic",
        action="store",
        required=True,
        type=str,
        help="The MQTT topic to publish the device's data to",
    )
    parser.add_argument(
        "--mqtt-port",
        action="store",
        default=DEFAULT_MQTT_PORT,
        type=int,
        help=f"The port of the MQTT broker (default: {DEFAULT_MQTT_PORT})",
    )
    parser.add_argument(
        "--mqtt-username",
        action="store",
        default=None,
        type=str,
        help="The username to use with the MQTT broker (default: None)",
    )
    parser.add_argument(
        "--mqtt-password",
        action="store",
        default=None,
        type=str,
        help="The password to use with the MQTT broker (default: None)",
    )
    parser.add_argument(
        "--endpoint",
        action="store",
        default=DEFAULT_AIOHTTP_ENDPOINT,
        type=str,
        help=(
            "The relative endpoint/path to serve the web app on "
            f"(default: {DEFAULT_AIOHTTP_ENDPOINT})"
        ),
    )
    parser.add_argument(
        "--port",
        action="store",
        default=DEFAULT_AIOHTTP_PORT,
        type=int,
        help=f"The port to serve the web app on (default: {DEFAULT_AIOHTTP_PORT})",
    )
    parser.add_argument(
        "-l",
        "--log-level",
        action="store",
        default=DEFAULT_LOG_LEVEL_STRING,
        help=f"The logging level (default: {DEFAULT_LOG_LEVEL_STRING})",
    )

    return parser.parse_args()


async def post_data(request: web.Request):
    """Define the endpoint for the Ecowitt device to post data to."""
    data = await request.post()

    _LOGGER.debug("Received data from Ecowitt device: %s", data)

    try:
        await request.app["mqtt"].publish(
            request.app["args"].mqtt_topic, json.dumps(dict(data)).encode()
        )
        _LOGGER.info("Data published to MQTT broker: %s", data)
    except MqttError as err:
        _LOGGER.error("Error while publishing data to MQTT: %s", err)


def main():
    """Run."""
    args = get_arguments()
    logging.basicConfig(level=getattr(logging, args.log_level))

    _LOGGER.debug("Using arguments: %s", args)

    app = web.Application()
    app.add_routes([web.post(args.endpoint, post_data)])
    app["args"] = args

    mqtt = app["mqtt"] = Client(
        args.mqtt_broker,
        port=args.mqtt_port,
        username=args.mqtt_username,
        password=args.mqtt_password,
    )

    async def connect_mqtt(_) -> None:
        """Connect the MQTT broker."""
        try:
            await mqtt.connect()
        except MqttError as err:
            _LOGGER.error("Error while connecting to MQTT broker: %s", err)

        _LOGGER.debug("Connected to MQTT broker")

    async def disconnect_mqtt(_) -> None:
        """Disconnect the MQTT broker."""
        try:
            await mqtt.disconnect()
        except MqttError as err:
            _LOGGER.error("Error while disconnecting from MQTT broker: %s", err)

        _LOGGER.debug("Disconnected from MQTT broker")

    app.on_startup.append(connect_mqtt)
    app.on_cleanup.append(disconnect_mqtt)

    web.run_app(app, port=args.port)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ecowitt2mqtt']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.6.2,<4.0.0',
 'asyncio-mqtt>=0.7.0,<0.8.0',
 'supervisor>=4.2.1,<5.0.0']

entry_points = \
{'console_scripts': ['ecowitt2mqtt = ecowitt2mqtt.main:main']}

setup_kwargs = {
    'name': 'ecowitt2mqtt',
    'version': '0.0.2',
    'description': 'A small web server to send data from Ecowitt devices to an MQTT Broker',
    'long_description': "# 🔘 ecowitt2mqtt: Send Ecowitt device data to an MQTT broker\n\n[![PyPi](https://img.shields.io/pypi/v/ecowitt2mqtt.svg)](https://pypi.python.org/pypi/ecowitt2mqtt)\n[![Version](https://img.shields.io/pypi/pyversions/ecowitt2mqtt.svg)](https://pypi.python.org/pypi/ecowitt2mqtt)\n[![License](https://img.shields.io/pypi/l/ecowitt2mqtt.svg)](https://github.com/bachya/ecowitt2mqtt/blob/master/LICENSE)\n[![Maintainability](https://api.codeclimate.com/v1/badges/a03c9e96f19a3dc37f98/maintainability)](https://codeclimate.com/github/bachya/ecowitt2mqtt/maintainability)\n[![Say Thanks](https://img.shields.io/badge/SayThanks-!-1EAEDB.svg)](https://saythanks.io/to/bachya)\n\n`ecowitt2mqtt` is a small CLI/web server that allows [Ecowitt](http://www.ecowitt.com)\ndevice data to be sent to an MQTT broker.\n\n- [Installation](#installation)\n- [Python Versions](#python-versions)\n- [Documentation](#documentation)\n- [Contributing](#contributing)\n\n# Installation\n\n```python\npip install ecowitt2mqtt\n```\n\n# Python Versions\n\n`ecowitt2mqtt` is currently supported on:\n\n* Python 3.6\n* Python 3.7\n* Python 3.8 \n\n# Usage\n\n## Command Line Interface\n\nThe library is controlled via an `ecowitt2mqtt` executable:\n\n```\nusage: ecowitt2mqtt [-h] --mqtt-broker MQTT_BROKER --mqtt-topic MQTT_TOPIC [--mqtt-port MQTT_PORT] [--mqtt-username MQTT_USERNAME]\n                    [--mqtt-password MQTT_PASSWORD] [--endpoint ENDPOINT] [--port PORT] [-l LOG_LEVEL]\n\nSend data from Ecowitt devices to an MQTT broker\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --mqtt-broker MQTT_BROKER\n                        The hostname or IP address of the MQTT broker\n  --mqtt-topic MQTT_TOPIC\n                        The MQTT topic to publish the device's data to\n  --mqtt-port MQTT_PORT\n                        The port of the MQTT broker (default: 1883)\n  --mqtt-username MQTT_USERNAME\n                        The username to use with the MQTT broker (default: None)\n  --mqtt-password MQTT_PASSWORD\n                        The password to use with the MQTT broker (default: None)\n  --endpoint ENDPOINT   The relative endpoint/path to serve the web app on (default: /data/report)\n  --port PORT           The port to serve the web app on (default: 8080)\n  -l LOG_LEVEL, --log-level LOG_LEVEL\n                        The logging level (default: INFO)\n```\nWhen run with the appropriate parameters, the executable will launch a web server with a\nsingle endpoint (with the default parameters, this endpoint will live at\n`http://0.0.0.0:8080/data/report`).\n\nWhen configured as a custom weather service in the WS View app, this endpoint will\nreceive data from the Ecowitt device and publish it to the defined MQTT broker.\n\n(TODO: flesh out these docs more)\n\n## Run in the Background\n\n`ecowitt2mqtt` doesn't, itself, provide any sort of daemonization mechanism. The suggested\nroute is to use something like [`supervisord`](http://www.supervisord.org):\n\n```\n[supervisord]\nnodaemon=true\nloglevel=info\nuser=root\n\n[program:ecowitt2mqtt]\ncommand=ecowitt2mqtt --mqtt-broker=192.168.1.100 --mqtt-topic=My/topic\nstdout_logfile=/dev/stdout\nstdout_logfile_maxbytes=0\nredirect_stderr=true\n```\n\n## Docker\n\nThe library is available via a Docker image (`bachya/ecowitt2mqtt`). It is configured by\na handful of environment variables that correspond to the command line parameters listed\nabove:\n\n* `LOG_LEVEL:` the log level to use (default: `INFO`)\n* `ENDPOINT:` the relative endpoint/path to serve the web app on (default: `/data/report`)\n* `PORT:` the port to serve the web app on (default: `8080`)\n* `MQTT_BROKER:` the hostname or IP address of the MQTT broker\n* `MQTT_PORT:` the port of the MQTT broker (default: `1883`)\n* `MQTT_PASSWORD:` the password to use with the MQTT broker (default: `None`)\n* `MQTT_USERNAME:` the password to use with the MQTT broker (default: `None`)\n* `MQTT_TOPIC:` the MQTT topic to publish the device's data to\n\nAn example `docker-compose` usage can be found in `docker-compose.dist.yml`.\n\n# Contributing\n\n1. [Check for open features/bugs](https://github.com/bachya/ecowitt2mqtt/issues)\n  or [initiate a discussion on one](https://github.com/bachya/ecowitt2mqtt/issues/new).\n2. [Fork the repository](https://github.com/bachya/ecowitt2mqtt/fork).\n3. (_optional, but highly recommended_) Create a virtual environment: `python3 -m venv .venv`\n4. (_optional, but highly recommended_) Enter the virtual environment: `source ./.venv/bin/activate`\n5. Install the dev environment: `script/setup`\n6. Code your new feature or bug fix.\n7. Write tests that cover your new functionality.\n8. Run tests and ensure 100% code coverage: `script/test`\n9. Update `README.md` with any new documentation.\n10. Add yourself to `AUTHORS.md`.\n11. Submit a pull request!\n",
    'author': 'Aaron Bach',
    'author_email': 'bachya1208@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bachya/ecowitt2mqtt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
